//+------------------------------------------------------------------+
//|                               PCA For Portfolio Optimization.mq5 |
//|                                        Gamuchirai Zororo Ndawana |
//|                          https://www.mql5.com/en/gamuchiraindawa |
//+------------------------------------------------------------------+
#property copyright "Gamuchirai Zororo Ndawana"
#property link      "https://www.mql5.com/en/gamuchiraindawa"
#property version   "1.00"

//+------------------------------------------------------------------+
//| Custom enumerations                                              |
//+------------------------------------------------------------------+
enum risk_level
  {
   High=0,
   Mid=1,
   Low=2
  };

//+------------------------------------------------------------------+
//| Libraries                                                        |
//+------------------------------------------------------------------+
#include  <Trade\Trade.mqh>
CTrade Trade;

//+------------------------------------------------------------------+
//| Constants values                                                 |
//+------------------------------------------------------------------+
const double high_risk_components[] = {#include "HIGH RISK COMPONENTS.txt"};
const double mid_risk_components[]  = {#include "MID RISK COMPONENTS.txt"};
const double low_risk_components[]  = {#include "LOW RISK COMPONENTS.txt"};
const string crypto[]               = {"BCHUSD","EOSUSD","BTCUSD","ETHUSD","ADAUSD","XRPUSD","UNIUSD","DOGUSD","LTCUSD","SOLUSD"};

//+------------------------------------------------------------------+
//| Global variables                                                 |
//+------------------------------------------------------------------+
double current_risk_settings[];
double vol,bid,ask;
int    atr_handler;
int    stoch_handler;
int    ma_handler;
double atr_reading[],ma_reading[],stoch_reading[];

//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
input group "Risk Levels"
input risk_level user_risk = High; //Which risk level should we use?

input group "Money Management"
input int lot_multiple     = 1;    //How big should out lot size be?

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Setup our market data
   setup();

//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- Release the indicators
   IndicatorRelease(atr_handler);
   IndicatorRelease(ma_handler);
   IndicatorRelease(stoch_handler);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- Ensure we always have 10 positions
   if(PositionsTotal() < 10)
     {
      //--- Let's see which market we aren't in
      for(int i = 0; i < 10; i++)
        {
         //--- Check if we can now enter that market
         if(!PositionSelect(crypto[i]))
           {
            check_setup(i);
           }
        }
     }
  }
//+------------------------------------------------------------------+
//| Update our system varaibles                                      |
//+------------------------------------------------------------------+
void update(string market)
  {
//--- Get current prices
   bid = SymbolInfoDouble(market,SYMBOL_BID);
   ask = SymbolInfoDouble(market,SYMBOL_ASK);
//--- Get current technical readings
   atr_handler   = iATR(market,PERIOD_CURRENT,14);
   stoch_handler = iStochastic(market,PERIOD_CURRENT,5,3,3,MODE_EMA,STO_CLOSECLOSE);
   ma_handler    = iMA(market,PERIOD_CURRENT,30,0,MODE_EMA,PRICE_CLOSE);
//--- Copy buffer
   CopyBuffer(atr_handler,0,0,1,atr_reading);
   CopyBuffer(ma_handler,0,0,1,ma_reading);
   CopyBuffer(stoch_handler,0,0,1,stoch_reading);
//---
  }

//+------------------------------------------------------------------+
//| Setup our market data                                            |
//+------------------------------------------------------------------+
void setup(void)
  {
//--- First let us define the current risk settings
   switch(user_risk)
     {
      //--- The user selected high risk
      case 0:
        {
         ArrayCopy(current_risk_settings,high_risk_components,0,0,WHOLE_ARRAY);
         Comment("EA in high risk mode");
         break;
        }
      //--- The user selected mid risk
      case 1:
        {
         ArrayCopy(current_risk_settings,mid_risk_components,0,0,WHOLE_ARRAY);
         Comment("EA in mid risk mode");
         break;
        }
      //--- The user selected low risk
      //--- Low risk is also the default setting for safety!
      default:
        {
         ArrayCopy(current_risk_settings,low_risk_components,0,0,WHOLE_ARRAY);
         Comment("EA in low risk mode");
         break;
        }
     }

  }

//+------------------------------------------------------------------+
//| Open a position if we can                                        |
//+------------------------------------------------------------------+
void check_setup(int idx)
  {
//--- The function takes the index of the symbol as its only parameter
//--- It will look up the principal component loading of the symbol to determine whether it should buy or sell
   update(crypto[idx]);
   vol = lot_multiple * SymbolInfoDouble(crypto[idx],SYMBOL_VOLUME_MIN);

   if(current_risk_settings[idx] > 0)
     {
      if((iClose(crypto[idx],PERIOD_D1,0) > ma_reading[0]) && (stoch_reading[0] > 50))
        {
         Comment("Analyzing: ",crypto[idx],"\nMA: ",ma_reading[0],"\nStoch: ",stoch_reading[0],"\nAction: Buy");
         Trade.Buy(vol,crypto[idx],ask,(ask - (atr_reading[0] * 3)),(ask + (atr_reading[0] * 3)),"PCA Risk Optimization");
         return;
        }
      else
        {
         Comment("Waiting for an oppurtunity to BUY: ",crypto[idx]);
        }
     }

   else
      if(current_risk_settings[idx] < 0)
        {

         if((iClose(crypto[idx],PERIOD_D1,0) < ma_reading[0]) && (stoch_reading[0] < 50))
           {
            Comment("Analyzing: ",crypto[idx],"\nClose: ","\nMA: ",ma_reading[0],"\nStoch: ",stoch_reading[0],"\nAction: Sell");
            Trade.Sell(vol,crypto[idx],bid,(bid + (atr_reading[0] * 3)),(bid - (atr_reading[0] * 3)),"PCA Risk Optimization");
            return;
           }

         else
           {
            Comment("Waiting for an oppurtunity to SELL: ",crypto[idx]);
            return;
           }
        }
   Comment("Analyzing: ",crypto[idx],"\nMA: ",ma_reading[0],"\nStoch: ",stoch_reading[0],"\nAction: None");
   return;
  };
//+------------------------------------------------------------------+
